#ifndef amanip_args_h
#define amanip_args_h

#include "smanip.h"

namespace amanip {

class args {
public:
  typedef std::pair<std::string,std::string> arg;
public:
  args(){}
  args(int a_argc,char* a_argv[]){
    for(int index=0;index<a_argc;index++) {
      std::vector<std::string> ws;
      words(a_argv[index],"=",false,ws);
      if(ws.size()==1) {
        m_args.push_back(arg(ws[0],"")); 
      } else if(ws.size()>=2) {
        m_args.push_back(arg(ws[0],ws[1])); 
      }
    }
  }
  virtual ~args(){}
  args(const args& a_from):m_args(a_from.m_args){}
  args& operator=(const args& a_from){m_args = a_from.m_args;return *this;}
public:
  inline bool is_arg(const std::string& a_string) const {
    for(std::vector<arg>::const_iterator it = m_args.begin();
        it!=m_args.end();++it) {
      if((*it).first==a_string) return true;
    }
    return false;
  }
  unsigned int number() const {return m_args.size();}
  inline bool find(const std::string& a_string,std::string& a_value) const {
    for(std::vector<arg>::const_iterator it = m_args.begin();
        it!=m_args.end();++it) {
      if((*it).first==a_string) {
        a_value = (*it).second;
        return true;
      }
    }
    a_value = "";
    return false;
  }
  template <class aT>
  inline bool find(const std::string& a_string,aT& a_value) const {
    std::string s;
    if(!find(a_string,s)) {a_value = 0;return false;}
    return to<aT>(s,a_value);
  }
  inline std::vector<std::string> tovector() const {
    // Return a vector of string <name=value>
    std::vector<std::string> vec;
    for(std::vector<arg>::const_iterator it = m_args.begin();
        it!=m_args.end();++it) {
      std::string s;
      if((*it).second.empty()) {
        s = (*it).first;
      } else {
        s = (*it).first;
        s += "=";
        s += (*it).second;
      }
      vec.push_back(s);
    }
    return vec;
  }
  inline bool add(const std::string& a_key,const std::string& a_value,bool a_override = true){
    if(a_override) {
      for(std::vector<arg>::iterator it = m_args.begin();
          it!=m_args.end();++it) {
        if((*it).first==a_key) {
          (*it).second = a_value;
          return true;
        }
      }
    }
    if(a_key.empty()) return false;
    m_args.push_back(arg(a_key,a_value)); 
    return true;
  }

  inline int remove(const std::string& a_key){
    unsigned int nbeg = m_args.size();
    for(std::vector<arg>::iterator it = m_args.begin();it!=m_args.end();) {
      if(a_key==(*it).first) {
        it = m_args.erase(it);
      } else {
        ++it;
      }
    }
    return nbeg - m_args.size();
  }
  inline std::vector<std::string> files(bool a_skip_first = true) const {
    // Get the serie of trailing args not beginning with '-'
    // and without a value (not of the form [-]xxx=yyy). 
    // Note that an argument like that in between arguments
    // not like that is NOT taken into account. 
    std::vector<std::string> files;
    if(m_args.empty()) return files;
    std::vector<arg>::const_iterator it = m_args.begin();
    if(a_skip_first) it++;
    for(;it!=m_args.end();++it) {
      if( ((*it).first.find('-')==0) || (*it).second.size() ) {
        files.clear();
      } else {
        files.push_back((*it).first);
      }
    }
    return files;
  }  
  inline bool argcv(int& a_argc,char**& a_argv) const {
    // If using with :
    //   int argc;
    //   char** argv;
    //   args.argcv(argc,argv);  
    // you can delete with :
    //   args.delete_argcv(argc,argv);
    if(m_args.empty()) {a_argc = 0;a_argv = 0;return true;}
    typedef char* cstring;
    cstring* av = new cstring[m_args.size()];
    if(!av) {a_argc = 0;a_argv = 0;return false;}
    a_argv = av;
    for(std::vector<arg>::const_iterator it = m_args.begin();
        it!=m_args.end();++it,av++) {
      std::string::size_type lf = (*it).first.length();
      std::string::size_type ls = (*it).second.length();
      std::string::size_type sz = 0;
      if(ls) {
        sz = lf + 1 + ls;
      } else {
        sz = lf;
      }
      char* p = new char[sz+1];
      if(!p) {a_argc = 0;a_argv = 0;return false;} //some delete are lacking.
      *av = p;
     {char* pf = (char*)(*it).first.c_str();
      for(std::string::size_type i=0;i<lf;i++) {*p++ = *pf++;}
      *p = 0;}
      if(ls) {*p = '=';p++;}
     {char* ps = (char*)(*it).second.c_str();
      for(std::string::size_type i=0;i<ls;i++) {*p++ = *ps++;}
      *p = 0;}
    }
    a_argc = (int)m_args.size();
    return true;
  }
  inline void delete_argcv(int& a_argc,char**& a_argv) const {
    for(int index=0;index<a_argc;index++) delete [] a_argv[index];
    delete [] a_argv;
    a_argc = 0;
    a_argv = 0;
  }
private:
  std::vector<arg> m_args;
};

inline std::string gui_toolkit(args& a_args){
  //used in oroot.
  std::string driver;
  a_args.find("-toolkit",driver);
  a_args.remove("-toolkit");
  if(driver.empty()) {
    if(a_args.is_arg("-Xt")||
       a_args.is_arg("-xt")||
       a_args.is_arg("-Xm")||
       a_args.is_arg("-xm")||
       a_args.is_arg("-Motif")||
       a_args.is_arg("-motif")) {
      driver = "Xt";
      a_args.remove("-Xt");
      a_args.remove("-xt");
      a_args.remove("-Xm");
      a_args.remove("-xm");
      a_args.remove("-Motif");
      a_args.remove("-motif");
    } else if(a_args.is_arg("-Win")||
              a_args.is_arg("-win")||
              a_args.is_arg("-Win32")||
              a_args.is_arg("-win32")) {
      driver = "Win";
      a_args.remove("-Win");
      a_args.remove("-win");
      a_args.remove("-Win32");
      a_args.remove("-win32");
    } else if(a_args.is_arg("-NextStep")||
              a_args.is_arg("-nextstep")) {
      driver = "NextStep";
      a_args.remove("-NextStep");
      a_args.remove("-nextstep");
    } else if(a_args.is_arg("-Gtk")||
              a_args.is_arg("-gtk")) {
      driver = "Gtk";
      a_args.remove("-Gtk");
      a_args.remove("-gtk");
    } else if(a_args.is_arg("-Qt")||
              a_args.is_arg("-qt")) {
      driver = "Qt";
      a_args.remove("-Qt");
      a_args.remove("-qt");
    } else if(a_args.is_arg("-SDL")||
              a_args.is_arg("-sdl")) {
      driver = "SDL";
      a_args.remove("-SDL");
      a_args.remove("-sdl");
    } else if(a_args.is_arg("-Net")||
              a_args.is_arg("-net")) {
      driver = "Net";
      a_args.remove("-Net");
      a_args.remove("-net");
    }
  }

  return driver;
}

}

#endif
